@@ -3,7 +3,7 @@
 #' spatially or temporally aggregate stars object, returning a data cube with lower spatial or temporal resolution
 #' 
 #' @param x object of class \code{stars} with information to be aggregated
-#' @param by object of class \code{sf} or \code{sfc} for spatial aggregation, for temporal aggregation a vector with time values (\code{Date}, \code{POSIXct}, or \code{PCICt}) that is interpreted as a sequence of left-closed, right-open time intervals or a string like "months", "5 days" or the like (see \link{cut.POSIXt}); if by is an object of class \code{stars}, it is converted to sfc by \code{st_as_sfc(by, as_points = FALSE)} thus ignoring its time component. Note: each pixel is assigned to only a single group (in the order the groups occur) so non-overlapping spatial features and temporal windows are recommended.
+#' @param by object of class \code{sf} or \code{sfc} for spatial aggregation, for temporal aggregation a vector with time values (\code{Date}, \code{POSIXct}, or \code{PCICt}) that is interpreted as a sequence of left-closed, right-open time intervals or a string like "months", "5 days" or the like (see \link{cut.POSIXt}), or a function that cuts time into intervals; if by is an object of class \code{stars}, it is converted to sfc by \code{st_as_sfc(by, as_points = FALSE)} thus ignoring its time component. Note: each pixel is assigned to only a single group (in the order the groups occur) so non-overlapping spatial features and temporal windows are recommended.
 #' @param FUN aggregation function, such as \code{mean}
 #' @param ... arguments passed on to \code{FUN}, such as \code{na.rm=TRUE}
 #' @param drop logical; ignored
@@ -76,17 +76,14 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 			paste(classes, collapse= ", "), "supported"))
 	if (inherits(by, "stars"))
 		by = st_as_sfc(by, as_points = FALSE) # and if not, then use st_normalize(by)
-	if (inherits(by, "sf"))
-		by = st_geometry(by) # sfc
 
 	if (inherits(by, "sf")) {
 		geom = attr(by, "sf_column")
 		by = st_geometry(by)
 	} else
 		geom = "geometry"
+	stopifnot(!missing(FUN), is.function(FUN))
 
-	if (missing(FUN))
-		stop("missing FUN argument")
 	if (exact && inherits(by, c("sf", "sfc_POLYGON", "sfc_MULTIPOLYGON")) && has_raster(x)) {
     	if (!requireNamespace("raster", quietly = TRUE))
         	stop("package raster required, please install it first") # nocov
@@ -149,7 +146,7 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 			ndims = 1
 			x = st_upfront(x, which_time(x))
 			values = expand_dimensions(x)[[1]]
-			if (inherits(by, "function")) {
+			if (is.function(by)) {
 				i = by(values)
 				if (!is.factor(i))
 					i = as.factor(i)
@@ -205,7 +202,7 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 
 	# reconstruct dimensions table:
 	d[[1]] = create_dimension(values = by)
-	names(d)[1] = if (inherits(by, c("POSIXct", "Date", "PCICt", "function")))
+	names(d)[1] = if (is.function(by) || inherits(by, c("POSIXct", "Date", "PCICt", "function")))
 			"time"
 		else
 			geom
@@ -234,13 +231,20 @@ aggregate.stars = function(x, by, FUN, ..., drop = FALSE, join = st_intersects,
 
 #' @export
 aggregate.stars_proxy = function(x, by, FUN, ...) {
-	if (inherits(by, "stars"))
-		by = st_as_sfc(by, as_points = FALSE)
-	if (!inherits(by, c("sf", "sfc", "sfg")))
-		stop("aggregate.stars_proxy only implemented for spatial `by' arguments")
-	by = st_geometry(by)
+	if (!inherits(by, c("sf", "sfc", "sfg", "stars")))
+		collect(x, match.call(), "aggregate", c("x", "by", "FUN"), env = environment(), ...)
+	else {
+		if (inherits(by, "stars"))
+			by = st_as_sfc(by, as_points = FALSE)
+		by = st_geometry(by)
 
-	# this assumes the result is small, no need to proxy
-	l = lapply(seq_along(by), function(i) aggregate(st_normalize(st_as_stars(x[by[i]])), by[i], FUN, ...))
-	do.call(c, c(l, along = list(which_sfc(l[[1]]))))
+		# this assumes each result of a [ selection is small enough to hold in memory
+		l = lapply(seq_along(by), 
+		   	function(i) {
+			   	sel_i = st_normalize(st_as_stars(x[by[i]]))
+			   	aggregate(sel_i, by[i], FUN, ...)
+		   	}
+		)
+		do.call(c, c(l, along = list(which_sfc(l[[1]]))))
+	}
 }
