@@ -4,7 +4,7 @@ maybe_normalizePath = function(.x, np = FALSE) {
 	if (is.function(.x) || !np || any(sapply(prefixes, has_prefix, x = .x)))
 		.x
 	else
-		normalizePath(.x, mustWork = FALSE)
+		setNames(normalizePath(.x, mustWork = FALSE), names(.x))
 }
 
 enc2utf8char = function(x) {
@@ -25,6 +25,26 @@ is_functions = function(x) {
 	is.function(x) || all(sapply(x, is.function))
 }
 
+is_big = function(x, ..., sub = sub, n_proxy) {
+	prod(dim(read_stars(x, ..., sub = sub, proxy = TRUE, quiet = TRUE))) > n_proxy
+}
+
+# strip common start & end parts of x, if remainder != ""
+unique_part = function(x, prefix) {
+	stopifnot(is.character(x), length(x) > 1)
+	original = x
+	while (nchar(x[1]) && length(unique(substr(x, 1, 1))) == 1)
+		x = sapply(x, substring, 2)
+	while ((n <- nchar(x[1])) && length(unique(substr(x, n, n))) == 1)
+		x = sapply(x, substring, 1, n - 1)
+	if (any(x == ""))
+		original
+	else if (is.character(prefix))
+		paste0(prefix, x)
+	else
+		x
+}
+
 
 #' read raster/array dataset from file or connection
 #'
@@ -39,14 +59,14 @@ is_functions = function(x) {
 #' @param RasterIO list with named parameters for GDAL's RasterIO, to further control the extent, resolution and bands to be read from the data source; see details.
 #' @param proxy logical; if \code{TRUE}, an object of class \code{stars_proxy} is read which contains array
 #' metadata only; if \code{FALSE} the full array data is read in memory. Always \code{FALSE} for curvilinear girds.
-#' If not set, defaults to \code{TRUE} when the number of cells to be read is larger than \code{options(stars.n_proxy},
-#' or to 1e8 if that option was not set.
+#' If set to a number, defaults to \code{TRUE} when the number of cells to be read is larger than that number.
 #' @param curvilinear length two character vector with names of subdatasets holding longitude and latitude values for all raster cells, or named length 2 list holding longitude and latitude matrices; the names of this list should correspond to raster dimensions referred to
 #' @param normalize_path logical; if \code{FALSE}, suppress a call to \link{normalizePath} on \code{.x}
 #' @param RAT character; raster attribute table column name to use as factor levels
 #' @param tolerance numeric; passed on to \link{all.equal} for comparing dimension parameters.
 #' @param ... passed on to \link{st_as_stars} if \code{curvilinear} was set
 #' @param exclude character; vector with category value(s) to exclude
+#' @param shorten logical or character; if \code{TRUE} and \code{length(.x) > 1}, remove common start and end parts of array names; if character a new prefix
 #' @return object of class \code{stars}
 #' @details In case \code{.x} contains multiple files, they will all be read and combined with \link{c.stars}. Along which dimension, or how should objects be merged? If \code{along} is set to \code{NA} it will merge arrays as new attributes if all objects have identical dimensions, or else try to merge along time if a dimension called \code{time} indicates different time stamps. A single name (or positive value) for \code{along} will merge along that dimension, or create a new one if it does not already exist. If the arrays should be arranged along one of more dimensions with values (e.g. time stamps), a named list can passed to \code{along} to specify them; see example.
 #'
@@ -95,12 +115,22 @@ is_functions = function(x) {
 #'   plot(st_as_sfc(red, as_points = FALSE), add = TRUE, border = 'green', lwd = 2)
 #' }
 #' file.remove(tmp)
-read_stars = function(.x, ..., options = character(0), driver = character(0),
-		sub = TRUE, quiet = FALSE, NA_value = NA_real_, along = NA_integer_,
-		RasterIO = list(), proxy = is_functions(.x) || (!length(curvilinear) &&
-				is_big(.x, sub = sub, driver=driver, normalize_path = normalize_path, ...)),
+read_stars = function(.x, sub = TRUE, ..., options = character(0), 
+		driver = character(0), quiet = FALSE, NA_value = NA_real_, along = NA_integer_,
+		RasterIO = list(), proxy = getOption("stars.n_proxy") %||% 1.e8,
 		curvilinear = character(0), normalize_path = TRUE, RAT = character(0),
-		tolerance = 1e-10, exclude = "") {
+		tolerance = 1e-10, exclude = "", shorten = TRUE) {
+
+	is_false = function(x) { length(x) == 1 && is.logical(x) && !x }
+	if (!is.list(.x) && length(.x) > 1 && is.null(names(.x)) && !is_false(shorten) &&
+				any((short_names <- unique_part(.x, shorten)) != .x))
+		names(.x) = short_names
+	if (is.numeric(proxy)) {
+		if (is_functions(.x))
+			proxy = TRUE
+		else if (length(curvilinear))
+			proxy = FALSE
+	}
 
 	x = if (is.list(.x)) {
 			f = function(y, np) enc2utf8char(maybe_normalizePath(y, np))
@@ -138,16 +168,16 @@ read_stars = function(.x, ..., options = character(0), driver = character(0),
 	if (length(curvilinear) && proxy)
 		warning("proxy = TRUE may not work for curvilinear rasters")
 
-	if (length(x) > 1) { # loop over data sources and RETURNS:
+	if (length(x) > 1) { # loop over data sources, read, c() all, and RETURN:
 		ret = lapply(x, read_stars, options = options, driver = driver, sub = sub, quiet = quiet,
 			NA_value = NA_value, RasterIO = as.list(RasterIO), proxy = proxy, curvilinear = curvilinear,
 			along = if (length(along) > 1) along[-1] else NA_integer_, normalize_path = normalize_path)
 		return(do.call(c, append(ret, list(along = along, tolerance = tolerance))))
 	}
 
-	# else:
+	# else: do it --
 	data = sf::gdal_read(get_names(x),
-		options = options, driver = driver, read_data = !proxy,
+		options = options, driver = driver, read_data = ifelse(is.logical(proxy), !proxy, proxy),
 		NA_value = NA_value, RasterIO_parameters = as.list(RasterIO))
 	if (!is.null(data$default_geotransform) && data$default_geotransform == 1) {
 		## we have the 0 1 0 0 0 1 transform indicated
@@ -205,6 +235,8 @@ read_stars = function(.x, ..., options = character(0), driver = character(0),
 				warning("only one array present: argument 'sub' will be ignored")
 		}
 		meta_data = structure(data, data = NULL) # take meta_data only
+		if (is.numeric(proxy) && !is.null(data$proxy))
+			proxy = data$proxy
 		data = if (proxy)
 				.x
 			else
@@ -251,26 +283,34 @@ read_stars = function(.x, ..., options = character(0), driver = character(0),
 				# select attribute table:
 				which.at = which(lengths(at) > 0)[1]
 				which.column = if (length(RAT))
-						RAT
+						RAT[1]
 					else
 						which(sapply(at[[which.at]], class) == "character")[1]
+				if (length(which.column) == 0)
+					which.column = 1
 				labels = at = at[[ which.at ]][[ which.column ]]
 				levels = 0:(length(at) - 1)
 				if (length(exclude)) {
 					ex = at %in% exclude
 					labels = labels[!ex]
 					levels = levels[!ex]
-					min_value = min(levels)
 					if (!is.null(co))
 						co = co[!ex]
 				} else
 					ex = rep(FALSE, length(levels))
-				f = factor(as.vector(data), levels = levels, labels = labels)
 			} else {
-				f = factor(as.vector(data))
-				ex = rep(FALSE, length(levels(f)))
+				levels = sort(unique(as.vector(data)))
+				labels = as.character(levels)
+				if (!is.null(co))
+					co = co[levels + 1]
+				ex = rep(FALSE, length(levels))
 			}
-			data = structure(f, dim = dim(data), colors = co, exclude = ex)
+			# f = factor(as.vector(data), levels = levels, labels = labels) # is too costly;
+			# see https://github.com/r-spatial/stars/issues/565:
+			# construct factor array manually:
+			data = structure(match(as.integer(as.vector(data)), levels),
+							 levels = labels, dim = dim(data), 
+							 colors = co, exclude = ex, class = "factor")
 		}
 
 		dims = if (proxy) {
@@ -295,9 +335,10 @@ read_stars = function(.x, ..., options = character(0), driver = character(0),
 				x
 		name_x = tail(strsplit(name_x, '[\\\\/:]+')[[1]], 1)
 		ret = if (proxy) { # no data present, subclass of "stars":
-				st_stars_proxy(setNames(list(x), names(.x) %||% name_x),
-					create_dimensions_from_gdal_meta(dims, meta_data), NA_value = NA_value,
-					resolutions = NULL, RasterIO = RasterIO)
+				dims = create_dimensions_from_gdal_meta(dims, meta_data)
+				st_stars_proxy(setNames(list(x), names(.x) %||% name_x), dims,
+					NA_value = NA_value, resolutions = NULL, RasterIO = RasterIO, 
+					file_dim = matrix(dim(dims), 1, dimnames = list(NULL, names(dims))))
 			} else
 				st_stars(setNames(list(data), names(.x) %||% name_x),
 					create_dimensions_from_gdal_meta(dim(data), meta_data))
@@ -309,15 +350,6 @@ read_stars = function(.x, ..., options = character(0), driver = character(0),
 	}
 }
 
-#' @export
-#' @name read_stars
-#' @param x object to be read with \link{read_stars}
-#' @param n_proxy integer; number of cells above which .x will be read as stars
-#' proxy object, i.e. not as in-memory arrays but left on disk
-is_big = function(x, ..., sub = sub, n_proxy = options("stars.n_proxy")[[1]] %||% 1.e8) {
-	prod(dim(read_stars(x, ..., sub = sub, proxy = TRUE, quiet = TRUE))) > n_proxy
-}
-
 get_data_units = function(data) {
 	units = unique(attr(data, "units")) # will fail parsing in as_units() when more than one
 	if (length(units) > 1) {
@@ -335,88 +367,3 @@ get_data_units = function(data) {
 			structure(data, units = NULL)
 	}
 }
-
-#' Read data using GDAL's multidimensional array API (experimental)
-#' 
-#' Read data using GDAL's multidimensional array API (experimental)
-#' @param x data source name
-#' @param variable name of the array to be read
-#' @param options array opening options
-#' @param raster names of the raster variables (default: first two)
-#' @param offset integer; offset for each dimension (pixels) of sub-array to read (default: 0,0,0,...) (requires sf >= 1.0-9)
-#' @param count integer; size for each dimension (pixels) of sub-array to read (default: read all) (requires sf >= 1.0-9)
-#' @param step integer; step size for each dimension (pixels) of sub-aray to read (requires sf >= 1.0-9)
-#' @param proxy logical; return proxy object? (not functional yet)
-#' @param debug logical; print debug info?
-#' @details it is assumed that the first two dimensions are easting / northing
-#' @param ... ignored
-#' @export
-read_mdim = function(x, variable = character(0), ..., options = character(0), raster = NULL,
-					 offset = integer(0), count = integer(0), step = integer(0), proxy = FALSE, 
-					 debug = FALSE) {
-#	ret = if (packageVersion("sf") >= "1.0-9")
-#			gdal_read_mdim(x, variable, options, rev(offset), rev(count), rev(step), proxy, debug)
-#		else
-#			gdal_read_mdim(x, variable, options)
-	if (packageVersion("sf") >= "1.0-9") {
-		message("update stars to > 0.5-6")
-		return(NULL)
-	}
-	ret = gdal_read_mdim(x, variable, options)
-	create_units = function(x) {
-		u <- attr(x, "units")
-		if (is.null(u) || u == "")
-			x
-		else {
-			if (!is.null(a <- attr(x, "attributes")) && !is.na(cal <- a["calendar"]) && 
-						cal %in% c("360_day", "365_day", "noleap"))
-				get_pcict(x, u, cal)
-			else {
-				u = units::set_units(x, u, mode = "standard")
-				p = try(as.POSIXct(u), silent = TRUE)
-				if (inherits(p, "POSIXct"))
-					p
-				else
-					u
-			}
-		}
-	}
-	l = rev(lapply(ret$dimensions, function(x) create_units(x$values[[1]])))
-	if (length(offset) != 0 || length(step) != 0 || length(count) != 0) {
-		if (length(offset) == 0)
-			offset = rep(0, length(l))
-		if (length(step) == 0)
-			step = rep(1, length(l))
-		if (length(count) == 0)
-			count = floor((lengths(l) - offset)/step)
-		for (i in seq_along(l)) {
-			l[[i]] = l[[i]][seq(from = offset[i]+1, length.out = count[i], by = step[i])]
-		}
-	}
-	d = mapply(function(x, i) create_dimension(values = x, is_raster = i %in% 1:2), l, seq_along(l),
-			SIMPLIFY = FALSE)
-	if (is.null(raster))
-		raster = get_raster(dimensions = names(d)[1:2])
-	else
-		raster = get_raster(dimensions = raster)
-
-	if (proxy)
-		stop("proxy not yet implemented in read_mdim()")
-
-	# handle array units:
-	for (i in seq_along(ret$array_list))
-		if (nchar(u <- attr(ret$array_list[[i]], "units")))
-			ret$array_list[[i]] = units::set_units(ret$array_list[[i]], u, mode = "standard")
-	lst = lapply(ret$array_list, function(x) structure(x, dim = rev(dim(x))))
-	st_set_crs(st_stars(lst, dimensions = structure(d, raster = raster, class = "dimensions")),
-		ret$srs)
-}
-
-write_mdim = function(x, filename, ...) {
-	stopifnot(inherits(x, "stars"))
-	to_units = function(x) if (inherits(x, c("POSIXct", "Date"))) units::as_units(x) else x
-	dimension_values = rev(lapply(expand_dimensions(x), to_units))
-	units = sapply(dimension_values, function(x) if(inherits(x, "units")) as.character(units(x)) else "")
-	gdal_write_mdim(st_as_stars(x), filename, dimension_values, units)
-	invisible(x)
-}
