@@ -7,6 +7,18 @@ st_as_sfc.stars = function(x, ..., as_points, which = seq_len(prod(dim(x)[1:2]))
 	r = attr(st_dimensions(x), "raster")
 	gt = get_geotransform(x)
 	d = st_dimensions(x)[r$dimensions]
+	if (xor(is.null(d[[1]]$values), is.null(d[[2]]$values))) {
+		# mixed regular/rectilinear dimensions: make rectilinear; https://github.com/r-spatial/stars/issues/458
+		e = expand_dimensions(d)
+		if (is.null(d[[1]]$values)) {
+			d[[1]]$from = d[[1]]$to = NA
+			d[[1]]$values = e[[1]]
+		}
+		if (is.null(d[[2]]$values)) {
+			d[[2]]$from = d[[2]]$to = NA
+			d[[2]]$values = e[[2]]
+		}
+	}
 	sfc = st_as_sfc(d, ..., as_points = as_points, which = which, geotransform = gt) 
 	# swap axes?
 	if (st_axis_order() && isTRUE(st_crs(x, parameters = TRUE)$yx))
@@ -36,6 +48,8 @@ st_xy2sfc = function(x, as_points, ..., na.rm = TRUE) {
 	dxy = attr(d, "raster")$dimensions
 	xy_pos = match(dxy, names(d))
 	stopifnot(all(xy_pos == 1:2))
+	if (missing(as_points) && isTRUE(d[[dxy[1]]]$point) && isTRUE(d[[dxy[2]]]$point))
+		as_points = TRUE
 
 	# find which records are NA for all attributes:
 	a = abind(x, along = length(dim(x)) + 1)
@@ -125,8 +139,15 @@ st_as_sf.stars = function(x, ..., as_points = FALSE, merge = FALSE, na.rm = TRUE
 				geotransform = get_geotransform(x), use_contours = FALSE, connect8 = connect8, ...)
 
 		# factor levels?
-		if (!is.null(lev <- attr(x[[1]], "levels")))
-			ret[[1]] = structure(ret[[1]], class = "factor", levels = lev)
+		if (!is.null(lev <- attr(x[[1]], "levels"))) {
+			ex = attr(x[[1]], "exclude")
+			if(any(ex)){
+				id = (seq_along(ex) - 1)[!ex] # index-values matching to levels
+				ret[[1]] = structure(match(ret[[1]], id), class = "factor", levels = lev)
+			} else {
+				ret[[1]] = structure(ret[[1]], class = "factor", levels = lev)
+			}
+		}
 		st_set_crs(ret, crs)
 	} else {
 		if (merge)
@@ -146,7 +167,9 @@ st_as_sf.stars = function(x, ..., as_points = FALSE, merge = FALSE, na.rm = TRUE
 				warning("working on the first sfc dimension only") # FIXME: this probably only works for 2D arrays, now
 			other_dim = setdiff(seq_along(dim(x)), ix[1])
 			sfc = st_dimensions(x)[[ ix[1] ]]$values
-			other_values = st_dimensions(x)[[ other_dim[1] ]]$values
+			# other_values = st_dimensions(x)[[ other_dim[1] ]]$values
+			other_values = lapply(st_dimensions(x)[other_dim], function(x) x$values)
+			varnames = apply(do.call(expand.grid, other_values), 1, paste, collapse = ".")
 			un_dim = function(x) { # remove a dim attribute from data.frame columns
 				for (i in seq_along(x))
 					x[[i]] = structure(x[[i]], dim = NULL)
@@ -158,8 +181,8 @@ st_as_sf.stars = function(x, ..., as_points = FALSE, merge = FALSE, na.rm = TRUE
 	
 			if (length(dim(x)) == 1) # one-dimensional cube...
 				names(df) = names(x)
-			else if (!is.null(other_values) && length(other_values) == ncol(df))
-				names(df) = other_values
+			else if (length(varnames) == ncol(df))
+				names(df) = varnames
 			else if (length(unique(names(df))) < ncol(df) && length(names(dfs)) == ncol(df)) # I hate this
 				names(df) = names(dfs)
 			else { # another exception... time as second dimension
